//code for the dummy bat used by team 16 B.A.T.S.S.
//code for making two ultrasonic sensors sense each other originally made by Aswinth Raj and then edited by team B.A.T.S.S. to fit our needs
//source: https://circuitdigest.com/microcontroller-projects/measuring-distance-between-two-ultrasonic-sensors-using-arduino

const int trigPin = 5, echoPin = 6, trigPin2 = 10, echoPin2 = 9;    //setting up the pinvalues used for the ultrasonic sensors, without a number is for the transmitting sensor, with a 2 means it is for the receiving part
const int PIN_RED = 4, PIN_GREEN = 3, PIN_BLUE = 2;   //setting the pinvalues for the RGB LEDs used for the bats eyes

long duration;    //setting up values that will be used to recognize being triggered
int distance, Pdistance;
int measuredDistance;

bool sending = true;    //setting up a sending boolean

unsigned long endTimeRecieving;   //preparing values to determine how long to delay the programme at certain points
unsigned long endTimeSending = millis() + 5000;
const int dataSendingInterval = 200;
unsigned long newDataSendTime = 0;

boolean hasReceived = false;    //setting up a boolean for when something has been received

void setup() {
  pinMode(trigPin, OUTPUT);   //setting the triggerpins to output and the echopins to input
  pinMode(echoPin, INPUT);
  pinMode(trigPin2, OUTPUT);
  pinMode(echoPin2, INPUT);

  pinMode(PIN_RED, OUTPUT);   //setting the RGB LED pins to output
  pinMode(PIN_GREEN, OUTPUT);
  pinMode(PIN_BLUE, OUTPUT);

  Serial.begin(9600);   //startign up the serial communication at 9600 baud
}

void loop() {
  while (!hasReceived) {    //while the bat has not received it will go between functions to receive or send depending on the sending boolean
    switch (sending) {
      case false:
        openToRecieve();
        break;
      case true:
        sendPulses();
        break;
    }
  }
  blinkEyes();  //the bats eyes will blink green if it has received something
}

void sendPulses() {
  setColor(255, 0, 0);    //when sendpulses is called the bat will show this by its eyes lighting up red
  while (endTimeSending > millis()) {     //starting a while loop for a certain timeframe
    digitalWrite(trigPin2, HIGH);   //while the while loop is active the bat will transmit using this code
    delayMicroseconds(10);
    digitalWrite(trigPin2, LOW);
    delay(2);
    sendData();   //the bat sends its information to the computer via the ESP
  }
  setColor(255, 255, 255);  //when done sending the bat will wait for a small amount of time as to make sure it does not receive itself, while that is the case the eyes will be white
  sending = false;
  endTimeRecieving = millis() + 10000;
  delay(1000);
}

void openToRecieve() {
  setColor(0, 0, 255);    //when the bat is receiving its eyes will light up blue
  sendData();   //the bat sends its data to the computer via the ESP
  Pdistance = distance;   //calculating the distance between sensors
  Calc();
  distance = duration * 0.034;
  if (Pdistance == distance || Pdistance == distance + 1 || Pdistance == distance - 1) {
    measuredDistance = distance / 2; 
    Serial.println(measuredDistance);
    if (measuredDistance >= 35 && measuredDistance <= 1000) {   //if the measuredDistance is between 35 and 1000 it is a useful variable, meaning that when sensed the bat has received something
      delay(10);
      hasRecieved();
    }
  }
  if (endTimeRecieving < millis()) {    //if the bat did not sense anything it will go back to sending mode
    sending = true;
    endTimeSending = millis() + 5000;
  }
  delay(12);
}

void hasRecieved() {    //if the bat received something it will serial print R1 and R0 to the ESP and turn hasReceived to true

  Serial.println("R1");
  delay(dataSendingInterval);
  Serial.println("R0");
  hasReceived = true;
}

void blinkEyes() {    //function to make the bats eyes blink
  setColor(0, 255, 0);
  delay(500);
  setColor(0, 0, 0);
  delay(500);
}

void Calc() {   //function used for the calculation of the distance between sensors
  duration = 0;
  Trigger_US();
  while (digitalRead(echoPin) == HIGH)
    ;
  delay(2);
  Trigger_US();
  duration = pulseIn(echoPin, HIGH);
}

void setColor(int R, int G, int B) {  //function to easily change the RGB LEDs color
  analogWrite(PIN_RED, R);
  analogWrite(PIN_GREEN, G);
  analogWrite(PIN_BLUE, B);
}

void Trigger_US() {   //function for triggering the triggerPin
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
}

void sendData() {   //function to send data to the EPS and therefore the computer
  if (newDataSendTime < millis()) {
    if (!sending) {
      Serial.println("S0");
    } else {
      Serial.println("S1");
    }
    newDataSendTime = millis() + dataSendingInterval;
  }
}