import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import processing.serial.*;

// Declare the Serial port, graphs and batcounter
Serial myPort = null;
Graph graphs[];
BatCounter batCounter;

// Declare all variables
JSONArray values;
HashMap<String, Integer> valHeaders = new HashMap<String, Integer>();
int lineColors[];
String valNames[];
boolean valTypes[];
int vars[];
int prevVars[];
int mathVarIndex = -1;
int mathTimeConst, counterTime = 0;
String buffer;
IntList batDetections = new IntList();

// Initilize the graphs, values and batcounter
void setup() {
  frameRate(25);
  size(1500, 1000);
  rectMode(CORNERS);
  imageMode(CENTER);
  comSelect(); // Let the user select the desired com port for serial connection
  values = loadJSONArray("Values.json"); // Load the Json Array containing all info on each variable
  vars = new int[values.size()]; // Initialize the vars, prevVars, valNames, lineColors and valTypes arrays to the size of the values json array
  prevVars  = new int[values.size()];
  valNames = new String[values.size()];
  lineColors = new int[values.size()];
  valTypes = new boolean[values.size()];
  for (int u = 0; u < values.size(); u++) { // For each value, store its data in their respective array for quick access by the program
    JSONObject val = values.getJSONObject(u);
    vars[u] = 0;
    prevVars[u] = 0;
    valNames[u] = val.getString("name");
    lineColors[u] = val.getInt("color");
    valTypes[u] = boolean(val.getInt("digital"));
    valHeaders.put(val.getString("header"), val.getInt("order")); // Hashmap to lookup what array index each variable has in vars[] by their header
    if (val.getInt("math")==1) { // If the variable is a math variable, set the mathVar index and mathTimeConst so to be able to known which variable the math operation should be stored in and calulated at what interval.
      mathVarIndex = val.getInt("order");
      mathTimeConst = val.getInt("time");
    }
    batCounter = new BatCounter(width-100, height-30);
  }
  JSONArray graphss;
  graphss = loadJSONArray("Graphs.json"); // Load the Json Array containing all info on each graph
  graphs = new Graph[graphss.size()]; // Initialize the graph array to the size of the graph json array
  for (int u = 0; u < graphss.size(); u++) {
    JSONObject graph = graphss.getJSONObject(u);
    if (graph.getInt("digitalvals")==1) { // If the graph is marked as digital, add a digital graph.
      graphs[u] = new DigitalGraph(graph.getInt("X1"), graph.getInt("Y1"), graph.getInt("X2"), graph.getInt("Y2"), graph.getInt("speed"), graph.getString("values"));
    } else { // If the graph is marked as not digital, add an analogue graph.
      graphs[u] = new AnalogueGraph(graph.getInt("X1"), graph.getInt("Y1"), graph.getInt("X2"), graph.getInt("Y2"), graph.getInt("speed"), graph.getString("values"), 0, graph.getInt("valuemax"));
    }
  }
}

// Set the com port to the passed name of the com port
void setComPort(String comname) {
  myPort = new Serial(this, comname, 9600);
}

// Display the graphs, call function to read the data, call function to calculate bat sightings in the last 10 seconds and display the bat counter
void draw() {
  if (myPort != null) { // If a serial port is initialized, call readData().
    readData();
    for (Graph g : graphs) {  // Display all graphs
      g.display();
    }
  }
  if (mathVarIndex != -1 && millis()>counterTime) { // At counterTime (10s) intervals, call batsOverTime().
    batsOverTime();
    counterTime += 100;
  }
  batCounter.display(); // Display the batCounter
}

// Read the Serial data
void readData() {
  if (myPort.available()>0) { // If serial data is available, read the data untill a newline, store in a buffer and trim the buffer to rid it of newline and carriage return.
    String buffer = myPort.readStringUntil('\n');
    if (buffer!=null) {
      buffer = buffer.trim();
      try { // Next, take the first character and try to match it to a key in the headers hashmap. If found, proceed. Else, catch the error.
        int index = valHeaders.get(str(buffer.charAt(0)));
        if (buffer.equals("Z1")) { // If the buffer is "Z1", indicating a bat is sighted, call batCounter.increaseCounter() and add the time of a bat sighting to the batDetections IntList.
          batDetections.append(millis());
          batCounter.increaseCounter();
        }
        if (valTypes[index]) { // Remove the first character (header) from the buffer and store int the values array vars. If the var is digital, map from 0 to 1, and if the var is analogue, map from 0 to 710, then get its corresponding graph and call graphs[i].mapval() function for the graph to map the value accordingly.
          vars[index] = int(buffer.substring(1));
        } else {
          vars[index] = int(map(int(buffer.substring(1)), 0, 1024, 0, 710));
          JSONObject val = values.getJSONObject(index);
          vars[index] = graphs[val.getInt("graph")].mapVal(int(buffer.substring(1)));
        }
      }
      catch (NullPointerException e) {
      }
      catch(StringIndexOutOfBoundsException e) {
      }
    }
  }
}

// If called, reset the math variable, loop over all batDetections, remove if expired and add a 1 to the variable that is a math variable when the sighting happened in the last 10 seconds.
void batsOverTime() {
  vars[mathVarIndex] = 0;
  int currTime = millis();
  for (int i = 0; i<batDetections.size(); i++) { // Loop over all bat Detections
    int time = batDetections.get(i);
    if (currTime>time) { // If the sighting happened in the last 10 seconds, increase the math variable representing bat sightings in the last 10 seconds
      if (currTime-time<mathTimeConst+5) {
        vars[mathVarIndex]++;
      } else {
        batDetections.remove(i); // Remove expired sighting
      }
    }
  }
}

// Generate a JFrame for the user to select the desired com port from which to read serial data. When the option is saved, call setComPort() to set the serial port to the selected com port.
void comSelect() {
  JFrame frame = new JFrame("Select"); // Create a new JFrame
  JButton button = new JButton("Save"); // Create a save button
  JComboBox coms = new JComboBox(Serial.list()); // Create a Combobox for the user to select the com port
  JLabel label = new JLabel("Select a com port", JLabel.CENTER); // Add an instructional text
  label.setSize(400, 100);
  frame.setResizable(false);
  coms.setBounds(50, 100, 90, 20);
  button.setBounds(200, 100, 75, 20);
  frame.add(coms); //Add all elements to the frame
  frame.add(button);
  frame.add(label);
  frame.setLayout(null);
  frame.setSize(350, 350);
  frame.setVisible(true);
  button.addActionListener( // Add actionlistener to the save button.
    new ActionListener() {
    public void actionPerformed(ActionEvent e) { // If the button is clicked, set the com port to the one selected by the user
      setComPort(coms.getSelectedItem().toString());
      frame.setVisible(false);
      frame.dispose();
    }
  }
  );
}

// When the program is exited, execute these functions to save the collected data.
void stop() {
  batCounter.writeCount();
  batCounter.savetable();
}
