class Graph {
  // Declare variables
  int graphingSpeed;
  int graphingPos;
  int posX2, posY2, posX, posY;
  int graphWindowX, graphWindowY, graphWindowX2, graphWindowY2;
  int windowSize;
  int vals[];

  // Initialize variables
  Graph(int posX, int posY, int posX2, int posY2, int speed, String values) {
    this. posX = posX;
    this. posY = posY;
    this. posX2 = posX2;
    this. posY2 = posY2;
    this.graphingSpeed = speed;
    vals = int(splitTokens(values, "o")); // Store which variables the graph should display
    graphWindowX = posX+20;
    graphWindowY = posY+20;
    graphWindowX2 = posX2-20;
    graphWindowY2 = posY+750;
    windowSize = graphWindowX2-graphWindowX;
    drawBackground();
  }

  // Display the graph
  void display() {
    if (graphingPos>graphWindowX2-5) { // If the position at which the graph draws the new lines representing the variables reaches the end of the graph, set it to back at the other end of the graph and remove its imprint from the screen.
      noStroke();
      fill(#06242E);
      rect(graphWindowX, graphWindowY, graphWindowX+5, graphWindowY2);
      graphingPos=graphWindowX+5;
    }
    drawGraph(); // Draw the new lines representing the variables
    graphingPos += graphingSpeed; // Move the position at which the graph draws the new lines representing the variables
    drawFade(); // Draw the fade to differentiate between new and old values
    noFill();
    stroke(0);
    rect(graphWindowX, graphWindowY, graphWindowX2, graphWindowY2);
  }

  // Draw the background of the graph
  void drawBackground() {
    fill(#043D52);
    stroke(#F28C2C);
    rect(posX, posY, posX2, posY2);
    stroke(0);
    fill(#06242E);
    rect(graphWindowX, graphWindowY, graphWindowX2, graphWindowY2);
    int textPos = graphWindowY2 + 50;
    fill(#FFFFFF);
    textSize(30);
    textAlign(LEFT, CENTER);
    text("Legend", 40+posX, textPos);
    textSize(20);
    textPos += 50;
    for (int val : vals) { // Draw the legend of the variables the graph draws
      JSONObject value = values.getJSONObject(val);
      text(value.getString("name"), 50+posX, textPos);
      colorMode(HSB);
      stroke(value.getInt("color"), 255, 255);
      strokeWeight(6);
      line(40+posX, textPos, 45+posX, textPos+4);
      colorMode(RGB);
      strokeWeight(2);
      textPos+=20;
    }
    graphingPos=graphWindowX+5;
  }

  // Draw a fade starting right from the graphing position and ending at the graphing position to differentiate the newly drawn variables more clearly
  void drawFade() {
    colorMode(RGB);
    noStroke();
    for (int i = 0; i<51; i++) {
      color c = color(6, 36, 46, 255-i*5);
      fill(c);
      if (i+graphingPos<graphWindowX2) {
        rect(graphingPos+i, graphWindowY, graphingPos+i+1, graphWindowY2);
      } else {
        rect(graphingPos+i-windowSize, graphWindowY, graphingPos+i-windowSize+1, graphWindowY2);
      }
    }
    if (!(graphingPos+5>graphWindowX2)) {
      fill(#FFFFFF);
      rect(graphingPos+1, graphWindowY+5, graphingPos+3, graphWindowY2-5);
    }
  }

  // Draw the graph, empty since subclasses draw their own different ways of displaying variables
  void drawGraph() {
  }

  // Map the given value accordingly to the data of this graph that says between what values to map. Then return the value
  int mapVal(int val) {
    return (val);
  }
}

class DigitalGraph extends Graph {

  DigitalGraph(int posX, int posY, int posX2, int posY2, int speed, String values) {
    super(posX, posY, posX2, posY2, speed, values);
  }

  // Draw the new lines representing the variables. Draw only the variables assigned to this graph
  void drawGraph() {
    noStroke();
    fill(#06242E);
    rect(graphingPos+1, graphWindowY+5, graphingPos+4, graphWindowY2-5);
    colorMode(HSB);
    for (int val : vals) {
      stroke(lineColors[val], 255, 255);
      line(graphingPos, (graphWindowY2-100)-500*prevVars[val], graphingPos+graphingSpeed, (graphWindowY2-100)-500*vars[val]);
      prevVars[val]=vars[val];
    }
  }
}

class AnalogueGraph extends Graph {
  int minVal, maxVal;

  AnalogueGraph(int posX, int posY, int posX2, int posY2, int speed, String values, int min, int max) {
    super(posX, posY, posX2, posY2, speed, values);
    this.minVal = min;
    this.maxVal = max;
  }

  // Draw the new lines representing the variables. Draw only the variables assigned to this graph
  void drawGraph() {
    noStroke();
    fill(#06242E);
    rect(graphingPos+1, graphWindowY+5, graphingPos+4, graphWindowY2-5);
    colorMode(HSB);
    for (int val : vals) {
      stroke(lineColors[val], 255, 255);
      line(graphingPos, (graphWindowY2-100)-prevVars[val], graphingPos+graphingSpeed, (graphWindowY2-100)-vars[val]);
      prevVars[val]=vars[val];
    }
  }

  // Map the given value accordingly to the data of this graph that says between what values to map. Then return the value
  int mapVal(int val) {
    println("aaa");
    return int(map(val, minVal, maxVal, 0, 710));
  }
}
