//code for the windmill used by team 16 B.A.T.S.S.
//code for making two ultrasonic sensors sense each other originally made by Aswinth Raj and then edited by team B.A.T.S.S. to fit our needs
//source: https://circuitdigest.com/microcontroller-projects/measuring-distance-between-two-ultrasonic-sensors-using-arduino

#include <SoftwareSerial.h>     //include the Softwareserial library to communicate to the ESP

const int trigPin = 11, echoPin = 10, trigPin2 = 6, echoPin2 = 7; //setting up constant integers for the sensor pins
const int PIN_LED = 8;    //setting up a constant integer the LED pin that lights up when the sensor has been triggered

long duration;        //setting up the variables for measuring the distance between sensors
int distance, Pdistance;
int measuredDistance;

bool sending = false;     //setting up the variables for when the windmill needs to send and receive
unsigned long endTimeSending;
const int dataSendingInterval = 100;
unsigned long newDataSendTime = 0;

char* z_value;    //setting up variables for the later transmitting of data to the computer
char* v_value;

#define RX 2       //defining the receiver pin on the Arduino to use for the ESP
#define TX 3        //defining the transmitter pin on the Arduino to use for the ESP
SoftwareSerial ESPserial(RX, TX);   //setting up the ESP

void setup() {
  pinMode(trigPin, OUTPUT);   //setting the echo pins to input and the trigger pins to output 
  pinMode(echoPin, INPUT);
  pinMode(trigPin2, OUTPUT);
  pinMode(echoPin2, INPUT);

  pinMode(PIN_LED, OUTPUT);   //setting the LED pin to output

  ESPserial.begin(38400);   //starting the communication with the ESP at 38400 baud
}

void loop() {
  switch (sending) {    //if sending is false, start receiving
    case false:
      openToRecieve();
      break;
    case true:    // if sending is true, send pulses
      sendPulses();
      break;
  }
  if (newDataSendTime < millis()) {   //if its time to send data, send data to ESP
    sendData();
  }
}

void openToRecieve() {
  digitalWrite(PIN_LED, LOW);   //when receiving the LED is off
  Pdistance = distance;   //calculating the distance between sensors
  Calc();
  distance = duration * 0.034;
  if (Pdistance == distance || Pdistance == distance + 1 || Pdistance == distance - 1) {
    measuredDistance = distance / 2;
    if (measuredDistance >= 35 && measuredDistance <= 1000) {   //if a useful signal was detected send to the ESP that you are sending and put sending on true
      delay(10);
      endTimeSending = millis() + 10000;    
      z_value = "Z1";
      ESPserial.println(z_value);
      Serial.println(z_value);
      sending = true;
    }
    measuredDistance = 0;   //reset measuredDistance
  }
  delay(12);
}

void sendPulses() {
  digitalWrite(PIN_LED, HIGH);    //the LED is on when sending
  while (endTimeSending > millis()) {   //the windmill will send pulses for a certain amount of time then turn sending to false
    digitalWrite(trigPin2, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPin2, LOW);
    delay(2);
  }
  sending = false;
  delay(1000);
}

void Calc() {     //function used for the calculation of the distance between sensors
  duration = 0;
  Trigger_US();
  while (digitalRead(echoPin) == HIGH)
    ;
  delay(2);
  Trigger_US();
  duration = pulseIn(echoPin, HIGH);
}

void Trigger_US() {     //function for triggering the triggerPin
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
}

void sendData() {     //function to send data to the EPS and therefore the computer, V0, V1 and Z0 are headers so the computer knows what information it is getting
  if (!sending) {
    v_value = "V0";
    ESPserial.println(v_value);
    Serial.println(v_value);
  } else {
    v_value = "V1";
    ESPserial.println(v_value);
    Serial.println(v_value);
  }
  z_value = "Z0";
  ESPserial.println(z_value);
  Serial.println(z_value);
  newDataSendTime += dataSendingInterval;
}