/*
  ESP RECIEVER to Serial port

  Receive data from an ESP32 sender via the ESP-NOW Wifi protocol.
  Retransmit the received data through the Serial connection.
*/

#include <esp_now.h>
#include <WiFi.h>


// Declare structure to receive data (matches sender structure)
typedef struct struct_message {
  char detectedData[32];
} struct_message;

// Create a struct_message called myData
struct_message myData;

// Callback function executed when data is received
void OnDataRecv(const uint8_t *mac, const uint8_t *incomingData, int len) {
  memcpy(&myData, incomingData, sizeof(myData));


  Serial.print(myData.detectedData);
}

void setup() {
  // Start serial connection
  Serial.begin(115200);

  // Set device as a Wifi Station
  WiFi.mode(WIFI_STA);

  // Initialize ESP-NOW
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  // After initiation register for callback to get packet status
  esp_now_register_recv_cb(OnDataRecv);
}

void loop() {
}
