/*
  ESP SENDER with Arduino input

  Transmit data received from the Arduino RX, TX pins to an
  ESP32 receiver using the ESP-NOW Wifi protocol.
*/


#include <esp_now.h>
#include <WiFi.h>

// Set MAC address of the receiver
uint8_t broadcastAddress[] = { 0x08, 0x3A, 0xF2, 0x71, 0xAE, 0x6C };  //08:3A:F2:71:AE:6C

// Declare RX, TX pins
#define RXp2 16
#define TXp2 17

// Declare buffer for Arduino data
char buffer[100];


// Setup ESP to ESP connection
typedef struct struct_message {
  char detected[32];
} struct_message;

// Create a struct_message called myData
struct_message myData;

esp_now_peer_info_t peerInfo;

// Callback when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
}


void setup() {
  // Start serial connection for displaying status
  Serial.begin(115200);

  // Start serial connection between Arduino RX, TX
  Serial2.begin(38400, SERIAL_8N1, RXp2, TXp2);
  WiFi.mode(WIFI_STA);

  // Initialize ESP-NOW protocol
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  // After initiation send callback to get packet status
  esp_now_register_send_cb(OnDataSent);

  // Register peer
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0;
  peerInfo.encrypt = false;

  // Add peer
  if (esp_now_add_peer(&peerInfo) != ESP_OK) {
    Serial.println("Failed to add peer");
    return;
  }
}
void loop() {
  if (Serial2.available() > 0) {
    int len = Serial2.readBytes(buffer, sizeof(buffer));
    buffer[len] = 0;
    char *str = buffer;
    Serial.println(str);

    // Write string to data structure
    strcpy(myData.detected, str);

    // Send data to the targeted MAC address
    esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *)&myData, sizeof(myData));

    if (result == ESP_OK) {
    } else {
      Serial.println("Error sending the data");
    }
  } else {
  }
}
